
require 'uberdoc'

module UberDoc
    
    class Task

        def initialize(options)
            @options = options
        end

        #
        #  Indicates if this task needs to run based on the options
        #
        def self.should_run?(options)
            true
        end

        #
        #  Performs the task
        #
        def perform
            puts "Subclass Responsibility"
        end # perform

        def self.known_tasks
            ObjectSpace.each_object(Class).select { |klass| klass < self }
        end

    end # Task

    #
    #  Returns a list of all known tasks
    #
    def self.find_all_tasks


        $LOAD_PATH.unshift(File.join(File.dirname(__FILE__), 'tasks'))
        Dir["#{File.dirname(__FILE__)}/tasks/**/*_task.rb"].each do |task|
            require task
        end
        
        Task.known_tasks

    end # find_all_tasks

    #
    #  Performs the specified tasks
    #
    def self.perform_tasks(tasks, options)
        
        performed = false
        
        tasks.each do |task|
            if task.should_run?(options)
                new_task = task.new(options)
                new_task.perform
                performed = true
            end
        end

        performed

    end # perform_tasks

    #
    #  Finds all appropriate tasks and performs them if necessary
    #
    def self.find_and_perform_tasks(options)
        self.perform_tasks(self.find_all_tasks, options)
    end

end # UberDoc
